#ifndef _DX_H_
#define _DX_H_

#include <ddraw.h>
#include <d3drm.h>

#include "dxerr.h"

// "g_" -> global
extern BOOL						g_bFullScreen;
extern BOOL						g_bActive;
extern BOOL						g_bUse3D;
extern LPDIRECTDRAW				g_pDD;				// DirectDraw object
extern LPDIRECTDRAWSURFACE		g_pDDSPrimary;		// Front surface
extern LPDIRECTDRAWSURFACE		g_pDDSBack;			// Back surface
extern LPDIRECTDRAWCLIPPER		g_pClipper;			// Clipper
extern HWND						g_hWnd;				// HWND
extern SIZE						g_szBack;
extern RECT						g_rcFront;

extern LPDIRECT3DRM				g_pD3DRM;			// Direct3DRM object
extern LPDIRECT3DRM				g_pD3DRM;			// Direct3DRM object
extern LPDIRECT3DRMDEVICE		g_pD3DRMDevice;
extern LPDIRECT3DRMVIEWPORT		g_pViewport;
extern LPDIRECT3DRMFRAME		g_pCamera;
extern LPDIRECT3DRMFRAME		g_pScene;

extern BOOL InitX(HWND hWnd, BOOL bUse3D = TRUE, BOOL bFullScreen = FALSE, int iX = 0, int iY = 0, int iDX = -1, int iDY = -1, BOOL bActive = FALSE);

extern HRESULT Init3D();

extern BOOL CreateDirect3DRM();
extern BOOL CreateDevice();
extern BOOL CreateViewport(D3DVALUE fBack = D3DVAL(4096.0));
extern BOOL CreateDefaultScene();
extern LPDIRECT3DRMFRAME LoadFrame(char* pszFilename);

extern HRESULT __cdecl loadTextures(char *name, void *arg, LPDIRECT3DRMTEXTURE *tex);

extern LPDIRECT3DRMFRAME GetFrameWithName(LPDIRECT3DRMFRAME pRootFrame, char* pszName);
extern char* GetFrameName(LPDIRECT3DRMFRAME pFrame);

// Set whether or not we should be redrawing
extern void SetActive(BOOL bActive);
// Determine whether or not we are active
extern BOOL GetActive();

// Set the pointers to initial values of NULL, etc.
extern void ClearX();

// Create the IDirectDraw object
extern HRESULT CreateDirectDraw();

// Set the cooperativelevel, based on g_bFullScreen
extern HRESULT DDSetCooperativeLevel(BOOL bFullScreen, HWND hWnd);

// Create a surface flipping structure
extern HRESULT CreateSurfaces(BOOL bUse3D, int iX = 0, int iY = 0, int iDX = -1, int iDY = -1);

// Set the display mode (for full-screen mode)
extern HRESULT DDSetDisplayMode(int iWidth = 0, int iHeight = 0, int iBpp = 0);

extern HRESULT DDResize(int iX, int iY, int iDX, int iDY);

// Flip surfaces if full-screen, else blit
extern HRESULT DDFlip();

// Switch between full-screen and windowed mode. Not working yet.
extern BOOL SwitchModes();

#endif
