
// comment this out if not an MFC application
#include "stdafx.h"

#include "dxerr.h"
#include "defs.h"

#include <mmsystem.h>
#include <ddraw.h>
#include <dsound.h>
#include <d3d.h>
#include <d3drm.h>
#include <d3drmwin.h>
#include <string.h>

// p == private
char* p_pszDefault = "Unrecognized error value.";

BOOL Failed(HRESULT hr)
{
	if (FAILED(hr)) {
//		LOG(ERRLOG, "Failed(): DirectX error: %s", ErrorToString(hr));
		TraceMsg("Failed", hr);
		return TRUE;
	}
	return FALSE;
}

BOOL Failed(HRESULT hr, char* pszMsg)
{
	if (FAILED(hr)) {
		TraceMsg(pszMsg, hr);
		return TRUE;
	}
	return FALSE;
}

void TraceMsg(char* pszMsg, HRESULT hr)
{
	TRACE("\n%s:%s\n", pszMsg, ErrorToString(hr));
}

char* ErrorToStringDS(HRESULT hr)
{
	switch(hr)
	{
	case DSERR_ALLOCATED : return "DSERR_ALLOCATED";
	case DSERR_CONTROLUNAVAIL : return "DSERR_CONTROLUNAVAIL";
	case DSERR_INVALIDPARAM : return "DSERR_INVALIDPARAM";
	case DSERR_INVALIDCALL : return "DSERR_INVALIDCALL";
	case DSERR_GENERIC : return "DSERR_GENERIC";
	case DSERR_PRIOLEVELNEEDED : return "DSERR_PRIOLEVELNEEDED";
	case DSERR_OUTOFMEMORY : return "DSERR_OUTOFMEMORY";
	case DSERR_BADFORMAT : return "DSERR_BADFORMAT";
	case DSERR_UNSUPPORTED : return "DSERR_UNSUPPORTED";
	case DSERR_NODRIVER : return "DSERR_NODRIVER";
	case DSERR_ALREADYINITIALIZED : return "DSERR_ALREADYINITIALIZED";
	case DSERR_NOAGGREGATION : return "DSERR_NOAGGREGATION";
	case DSERR_BUFFERLOST : return "DSERR_BUFFERLOST";
	case DSERR_OTHERAPPHASPRIO : return "DSERR_OTHERAPPHASPRIO";
	case DSERR_UNINITIALIZED : return "DSERR_UNINITIALIZED";
	}
	return p_pszDefault;
}

char* ErrorToStringDD(HRESULT hr)
{
	switch(hr)
	{
	case DD_OK: return "DD_OK:No error.";
	case DDERR_ALREADYINITIALIZED: return "DDERR_ALREADYINITIALIZED:This object is already initialized.";
	case DDERR_BLTFASTCANTCLIP: return "DDERR_BLTFASTCANTCLIP:Return if a clipper object is attached to the source surface passed into a BltFast call.";
	case DDERR_CANNOTATTACHSURFACE: return "DDERR_CANNOTATTACHSURFACE:This surface can not be attached to the requested surface.";
	case DDERR_CANNOTDETACHSURFACE: return "DDERR_CANNOTDETACHSURFACE:This surface can not be detached from the requested surface.";
	case DDERR_CANTCREATEDC: return "DDERR_CANTCREATEDC:Windows can not create any more DCs.";
	case DDERR_CANTDUPLICATE: return "DDERR_CANTDUPLICATE:Can't duplicate primary & 3D surfaces, or surfaces that are implicitly created.";
	case DDERR_CLIPPERISUSINGHWND: return "DDERR_CLIPPERISUSINGHWND:An attempt was made to set a cliplist for a clipper object that is already monitoring an hwnd.";
	case DDERR_COLORKEYNOTSET: return "DDERR_COLORKEYNOTSET:No src color key specified for this operation.";
	case DDERR_CURRENTLYNOTAVAIL: return "DDERR_CURRENTLYNOTAVAIL:Support is currently not available.";
	case DDERR_DIRECTDRAWALREADYCREATED: return "DDERR_DIRECTDRAWALREADYCREATED:A DirectDraw object representing this driver has already been created for this process.";
	case DDERR_EXCEPTION: return "DDERR_EXCEPTION:An exception was encountered while performing the requested operation.";
	case DDERR_EXCLUSIVEMODEALREADYSET: return "DDERR_EXCLUSIVEMODEALREADYSET:An attempt was made to set the cooperative level when it was already set to exclusive.";
	case DDERR_GENERIC: return "DDERR_GENERIC:Generic failure.";
	case DDERR_HEIGHTALIGN: return "DDERR_HEIGHTALIGN:Height of rectangle provided is not a multiple of reqd alignment.";
	case DDERR_HWNDALREADYSET: return "DDERR_HWNDALREADYSET:The CooperativeLevel HWND has already been set. It can not be reset while the process has surfaces or palettes created.";
	case DDERR_HWNDSUBCLASSED: return "DDERR_HWNDSUBCLASSED:HWND used by DirectDraw CooperativeLevel has been subclassed, this prevents DirectDraw from restoring state.";
	case DDERR_IMPLICITLYCREATED: return "DDERR_IMPLICITLYCREATED:This surface can not be restored because it is an implicitly created surface.";
	case DDERR_INCOMPATIBLEPRIMARY: return "DDERR_INCOMPATIBLEPRIMARY:Unable to match primary surface creation request with existing primary surface.";
	case DDERR_INVALIDCAPS: return "DDERR_INVALIDCAPS:One or more of the caps bits passed to the callback are incorrect.";
	case DDERR_INVALIDCLIPLIST: return "DDERR_INVALIDCLIPLIST:DirectDraw does not support the provided cliplist.";
	case DDERR_INVALIDDIRECTDRAWGUID: return "DDERR_INVALIDDIRECTDRAWGUID:The GUID passed to DirectDrawCreate is not a valid DirectDraw driver identifier.";
	case DDERR_INVALIDMODE: return "DDERR_INVALIDMODE:DirectDraw does not support the requested mode.";
	case DDERR_INVALIDOBJECT: return "DDERR_INVALIDOBJECT:DirectDraw received a pointer that was an invalid DIRECTDRAW object.";
	case DDERR_INVALIDPARAMS: return "DDERR_INVALIDPARAMS:One or more of the parameters passed to the function are incorrect.";
	case DDERR_INVALIDPIXELFORMAT: return "DDERR_INVALIDPIXELFORMAT:The pixel format was invalid as specified.";
	case DDERR_INVALIDPOSITION: return "DDERR_INVALIDPOSITION:Returned when the position of the overlay on the destination is no longer legal for that destination.";
	case DDERR_INVALIDRECT: return "DDERR_INVALIDRECT:Rectangle provided was invalid.";
	case DDERR_LOCKEDSURFACES: return "DDERR_LOCKEDSURFACES:Operation could not be carried out because one or more surfaces are locked.";
	case DDERR_NO3D: return "DDERR_NO3D:There is no 3D present.";
	case DDERR_NOALPHAHW: return "DDERR_NOALPHAHW:Operation could not be carried out because there is no alpha accleration hardware present or available.";
	case DDERR_NOBLTHW: return "DDERR_NOBLTHW:No blitter hardware present.";
	case DDERR_NOCLIPLIST: return "DDERR_NOCLIPLIST:No cliplist available.";
	case DDERR_NOCLIPPERATTACHED: return "DDERR_NOCLIPPERATTACHED:No clipper object attached to surface object.";
	case DDERR_NOCOLORCONVHW: return "DDERR_NOCOLORCONVHW:Operation could not be carried out because there is no color conversion hardware present or available.";
	case DDERR_NOCOLORKEY: return "DDERR_NOCOLORKEY:Surface doesn't currently have a color key";
	case DDERR_NOCOLORKEYHW: return "DDERR_NOCOLORKEYHW:Operation could not be carried out because there is no hardware support of the destination color key.";
	case DDERR_NOCOOPERATIVELEVELSET: return "DDERR_NOCOOPERATIVELEVELSET:Create function called without DirectDraw object method SetCooperativeLevel being called.";
	case DDERR_NODC: return "DDERR_NODC:No DC was ever created for this surface.";
	case DDERR_NODDROPSHW: return ":No DirectDraw ROP hardware.";
	case DDERR_NODIRECTDRAWHW: return ":A hardware-only DirectDraw object creation was attempted but the driver did not support any hardware.";
	case DDERR_NOEMULATION: return ":Software emulation not available.";
	case DDERR_NOEXCLUSIVEMODE: return ":Operation requires the application to have exclusive mode but the application does not have exclusive mode.";
	case DDERR_NOFLIPHW: return ":Flipping visible surfaces is not supported.";
	case DDERR_NOGDI: return ":There is no GDI present.";
	case DDERR_NOHWND: return ":Clipper notification requires an HWND or no HWND has previously been set as the CooperativeLevel HWND.";
	case DDERR_NOMIRRORHW: return ":Operation could not be carried out because there is no hardware present or available.";
	case DDERR_NOOVERLAYDEST: return ":Returned when GetOverlayPosition is called on an overlay that UpdateOverlay has never been called on to establish a destination.";
	case DDERR_NOOVERLAYHW: return ":Operation could not be carried out because there is no overlay hardware present or available.";
	case DDERR_NOPALETTEATTACHED: return ":No palette object attached to this surface.";
	case DDERR_NOPALETTEHW: return ":No hardware support for 16 or 256 color palettes.";
	case DDERR_NORASTEROPHW: return ":Operation could not be carried out because there is no appropriate raster op hardware present or available.";
	case DDERR_NOROTATIONHW: return "DDERR_NOROTATIONHW:Operation could not be carried out because there is no rotation hardware present or available.";
	case DDERR_NOSTRETCHHW: return "DDERR_NOSTRETCHHW:Operation could not be carried out because there is no hardware support for stretching.";
	case DDERR_NOT4BITCOLOR: return "DDERR_NOT4BITCOLOR:DirectDrawSurface is not in 4 bit color palette and the requested operation requires 4 bit color palette.";
	case DDERR_NOT4BITCOLORINDEX: return "DDERR_NOT4BITCOLORINDEX:DirectDrawSurface is not in 4 bit color index palette and the requested operation requires 4 bit color index palette.";
	case DDERR_NOT8BITCOLOR: return "DDERR_NOT8BITCOLOR:DirectDrawSurface is not in 8 bit color mode and the requested operation requires 8 bit color.";
	case DDERR_NOTAOVERLAYSURFACE: return "DDERR_NOTAOVERLAYSURFACE:Returned when an overlay member is called for a non-overlay surface.";
	case DDERR_NOTEXTUREHW: return "DDERR_NOTEXTUREHW:Operation could not be carried out because there is no texture mapping hardware present or available.";
	case DDERR_NOTFLIPPABLE: return "DDERR_NOTFLIPPABLE:An attempt has been made to flip a surface that is not flippable.";
	case DDERR_NOTFOUND: return "DDERR_NOTFOUND:Requested item was not found.";
	case DDERR_NOTLOCKED: return "DDERR_NOTLOCKED:Surface was not locked.  An attempt to unlock a surface that was not locked at all, or by this process, has been attempted.";
	case DDERR_NOTPALETTIZED: return "DDERR_NOTPALETTIZED:The surface being used is not a palette-based surface.";
	case DDERR_NOVSYNCHW: return "DDERR_NOVSYNCHW:Operation could not be carried out because there is no hardware support for vertical blank synchronized operations.";
	case DDERR_NOZBUFFERHW: return ":Operation could not be carried out because there is no hardware support for zbuffer blitting.";
	case DDERR_NOZOVERLAYHW: return ":Overlay surfaces could not be z layered based on their BltOrder because the hardware does not support z layering of overlays.";
	case DDERR_OUTOFCAPS: return ":The hardware needed for the requested operation has already been allocated.";
	case DDERR_OUTOFMEMORY: return ":DirectDraw does not have enough memory to perform the operation.";
	case DDERR_OUTOFVIDEOMEMORY: return ":DirectDraw does not have enough memory to perform the operation.";
	case DDERR_OVERLAYCANTCLIP: return ":The hardware does not support clipped overlays.";
	case DDERR_OVERLAYCOLORKEYONLYONEACTIVE: return "DDERR_OVERLAYCOLORKEYONLYONEACTIVE:Can only have ony color key active at one time for overlays.";
	case DDERR_OVERLAYNOTVISIBLE: return "DDERR_OVERLAYNOTVISIBLE:Returned when GetOverlayPosition is called on a hidden overlay.";
	case DDERR_PALETTEBUSY: return ":Access to this palette is being refused because the palette is already locked by another thread.";
	case DDERR_PRIMARYSURFACEALREADYEXISTS: return ":This process already has created a primary surface.";
	case DDERR_REGIONTOOSMALL: return ":Region passed to Clipper::GetClipList is too small.";
	case DDERR_SURFACEALREADYATTACHED: return ":This surface is already attached to the surface it is being attached to.";
	case DDERR_SURFACEALREADYDEPENDENT: return ":This surface is already a dependency of the surface it is being made a dependency of.";
	case DDERR_SURFACEBUSY: return ":Access to this surface is being refused because the surface is already locked by another thread.";
	case DDERR_SURFACEISOBSCURED: return ":Access to surface refused because the surface is obscured.";
	case DDERR_SURFACELOST: return ":Access to this surface is being refused because the surface memory is gone. The DirectDrawSurface object representing this surface should have Restore called on it.";
	case DDERR_SURFACENOTATTACHED: return ":The requested surface is not attached.";
	case DDERR_TOOBIGHEIGHT: return ":Height requested by DirectDraw is too large.";
	case DDERR_TOOBIGSIZE: return ":Size requested by DirectDraw is too large, but the individual height and width are OK.";
	case DDERR_TOOBIGWIDTH: return ":Width requested by DirectDraw is too large.";
	case DDERR_UNSUPPORTED: return ":Action not supported.";
	case DDERR_UNSUPPORTEDFORMAT: return "DDERR_UNSUPPORTEDFORMAT:FOURCC format requested is unsupported by DirectDraw.";
	case DDERR_UNSUPPORTEDMASK: return ":Bitmask in the pixel format requested is unsupported by DirectDraw.";
	case DDERR_VERTICALBLANKINPROGRESS: return "DDERR_VERTICALBLANKINPROGRESS:Vertical blank is in progress.";
	case DDERR_WASSTILLDRAWING: return "DDERR_WASSTILLDRAWING:Informs DirectDraw that the previous Blt which is transfering information to or from this Surface is incomplete.";
	case DDERR_WRONGMODE: return "DDERR_WRONGMODE:This surface can not be restored because it was created in a different mode.";
	case DDERR_XALIGN: return "DDERR_XALIGN:Rectangle provided was not horizontally aligned on required boundary.";
	default:
		return p_pszDefault;
	}
}

char* ErrorToStringD3D(HRESULT hr)
{
	switch(hr)
	{
	case D3DERR_BADMAJORVERSION: return "D3DERR_BADMAJORVERSION";
	case D3DERR_BADMINORVERSION: return "D3DERR_BADMINORVERSION";
	case D3DERR_EXECUTE_LOCKED: return "D3DERR_EXECUTE_LOCKED";
	case D3DERR_EXECUTE_NOT_LOCKED: return "D3DERR_EXECUTE_NOT_LOCKED";
	case D3DERR_EXECUTE_CREATE_FAILED: return "D3DERR_EXECUTE_CREATE_FAILED";
	case D3DERR_EXECUTE_DESTROY_FAILED: return "D3DERR_EXECUTE_DESTROY_FAILED";
	case D3DERR_EXECUTE_LOCK_FAILED: return "D3DERR_EXECUTE_LOCK_FAILED";
	case D3DERR_EXECUTE_UNLOCK_FAILED: return "D3DERR_EXECUTE_UNLOCK_FAILED";
	case D3DERR_EXECUTE_FAILED: return "D3DERR_EXECUTE_FAILED";
	case D3DERR_EXECUTE_CLIPPED_FAILED: return "D3DERR_EXECUTE_CLIPPED_FAILED";
	case D3DERR_TEXTURE_NO_SUPPORT: return "D3DERR_TEXTURE_NO_SUPPORT";
	case D3DERR_TEXTURE_NOT_LOCKED: return "D3DERR_TEXTURE_NOT_LOCKED";
	case D3DERR_TEXTURE_LOCKED: return "D3DERR_TEXTURELOCKED";
	case D3DERR_TEXTURE_CREATE_FAILED: return "D3DERR_TEXTURE_CREATE_FAILED";
	case D3DERR_TEXTURE_DESTROY_FAILED: return "D3DERR_TEXTURE_DESTROY_FAILED";
	case D3DERR_TEXTURE_LOCK_FAILED: return "D3DERR_TEXTURE_LOCK_FAILED";
	case D3DERR_TEXTURE_UNLOCK_FAILED: return "D3DERR_TEXTURE_UNLOCK_FAILED";
	case D3DERR_TEXTURE_LOAD_FAILED: return "D3DERR_TEXTURE_LOAD_FAILED";
	case D3DERR_MATRIX_CREATE_FAILED: return "D3DERR_MATRIX_CREATE_FAILED";
	case D3DERR_MATRIX_DESTROY_FAILED: return "D3DERR_MATRIX_DESTROY_FAILED";
	case D3DERR_MATRIX_SETDATA_FAILED: return "D3DERR_MATRIX_SETDATA_FAILED";
	case D3DERR_SETVIEWPORTDATA_FAILED: return "D3DERR_SETVIEWPORTDATA_FAILED";
	case D3DERR_MATERIAL_CREATE_FAILED: return "D3DERR_MATERIAL_CREATE_FAILED";
	case D3DERR_MATERIAL_DESTROY_FAILED: return "D3DERR_MATERIAL_DESTROY_FAILED";
	case D3DERR_MATERIAL_SETDATA_FAILED: return "D3DERR_MATERIAL_SETDATA_FAILED";
	case D3DERR_LIGHT_SET_FAILED: return "D3DERR_LIGHT_SET_FAILED";
	case D3DRMERR_BADOBJECT: return "D3DRMERR_BADOBJECT";
	case D3DRMERR_BADTYPE: return "D3DRMERR_BADTYPE";
	case D3DRMERR_BADALLOC: return "D3DRMERR_BADALLOC";
	case D3DRMERR_FACEUSED: return "D3DRMERR_FACEUSED";
	case D3DRMERR_NOTFOUND: return "D3DRMERR_NOTFOUND";
	case D3DRMERR_NOTDONEYET: return "D3DRMERR_NOTDONEYET";
	case D3DRMERR_FILENOTFOUND: return "D3DRMERR_FILENOTFOUND:The file was not found.";
	case D3DRMERR_BADFILE: return "D3DRMERR_BADFILE";
	case D3DRMERR_BADDEVICE: return "D3DRMERR_BADDEVICE";
	case D3DRMERR_BADVALUE: return "D3DRMERR_BADVALUE";
	case D3DRMERR_BADMAJORVERSION: return "D3DRMERR_BADMAJORVERSION";
	case D3DRMERR_BADMINORVERSION: return "D3DRMERR_BADMINORVERSION";
	case D3DRMERR_UNABLETOEXECUTE: return "D3DRMERR_UNABLETOEXECUTE";
		
	default:
		return p_pszDefault;
	}
}

char* ErrorToString(HRESULT hr)
{
	char* pszRet;
	
	pszRet = ErrorToStringDD(hr);
	if (strcmp(pszRet, p_pszDefault) != 0)
		return pszRet;

	pszRet = ErrorToStringD3D(hr);
	if (strcmp(pszRet, p_pszDefault) != 0)
		return pszRet;

	pszRet = ErrorToStringDS(hr);
	if (strcmp(pszRet, p_pszDefault) != 0)
		return pszRet;

	return p_pszDefault;
}
